;##########################################################################

; version d [14/10/03]


;GRASS EMITTER
Type grass

Field id%
Field hndl.girt
Field tris%
Field tex$
Field grassmap$
Field texflag%,fx%,blend%,alpha#
Field r%,g%,b%;for vertex coloring

Field size#;size of the plant
Field size2#;if >0 then varie size between size# and size2#

Field z$;the z-order bin file to use

End Type





;grass type emitter type

Type girt
	Field	id
	Field	mesh
	Field   x#,y#,z#
	Field	surf
	Field	cache.adirt
	Field	pos.adirt
	Field	texture
End Type


;==========================================================================



;COMMON

Type vector
	Field id%
	Field	x#
	Field	y#
	Field	z#
End Type



Type adirt
	Field	id
	Field	link.adirt
	Field	index
	Field	pos.vector
	Field	size#,size2#
End Type



;Lookup Tables dims
Dim lcos#(0)
Dim lsin#(0)






;##########################################################################
;
;GRASS TYPE EMITTER FUNCTIONS
;
;##########################################################################







;Create the emitters
Function GRASS_MAKE(terrain,terscale#)

Local x#,y#,z#,gw%,gh%,divx%,divz%,g_r%,g_g%,g_b%,rgb%,grasm%,buf%

;stretch along the x,z coords of the terrain
Local tsize%=TerrainSize(terrain)
Local tsc#=tsize*terscale#
Local grx1#=EntityX(terrain)
Local grx2#=EntityX(terrain)+tsc
Local gry1#=0
Local gry2#=0
Local grz1#=EntityZ(terrain)
Local grz2#=EntityZ(terrain)+tsc


buf=GraphicsBuffer()


For gr.grass=Each grass

;if a grassmap is declared
If gr\grassmap$<>"" And FileType(gr\grassmap$)=1
grasm=LoadImage(gr\grassmap$);
flip_vertical(grasm)
gw=ImageWidth(grasm)
gh=ImageHeight(grasm)
SetBuffer(ImageBuffer(grasm))
LockBuffer(ImageBuffer(grasm))
EndIf


gr\hndl=Create_grass(gr\id,(grx1+grx2)/2.0,(gry1+gry2)/2.0,(grz1+grz2)/2.0,gr\tris,gr\tex$,gr\texflag,gr\fx,gr\blend,gr\alpha)

surf=GetSurface(gr\hndl\mesh,1)

x=EntityX(gr\hndl\mesh)
y=EntityY(gr\hndl\mesh)
z=EntityZ(gr\hndl\mesh)



For i=1 To gr\tris
;----------------------

.retry

If grasm=0
tx#=x+Rnd(grx1,grx2)
tz#=z+Rnd(grz1,grz2)
Else
divx%=Rnd(0,gw-1)
divz%=Rnd(0,gh-1)
EndIf


If grasm<>0

rgb=ReadPixelFast(divx,divz,ImageBuffer(grasm)) And $FFFFFF
g_r=(rgb Shr 16 And $FF)
g_g=(rgb Shr 8 And $FF)
g_b=(rgb And $FF)

If g_g=g_r And g_g=g_b Goto retry

tx#=x+divx*terscale#+grx1
tz#=z+divz*terscale#+grz1

EndIf

;--------------------

Add_grass(gr\hndl,gr\size,gr\size2,tx,TerrainY(terrain,tx,0,tz),tz)

Next



Update_grass(gr\hndl,terrain,gr\r,gr\g,gr\b)

UpdateNormals gr\hndl\mesh

If grasm<>0
UnlockBuffer(ImageBuffer(grasm))
FreeImage grasm:grasm=0
EndIf


;-----------------------------------------------
;ZORDERIZE
zo.zorder = Zorderize(gr\hndl\mesh,gr\id,0,gr\z$)
gr\hndl\mesh = zo\screenmesh
gr\hndl\surf = GetSurface(zo\screenmesh,1)
EntityTexture zo\screenmesh,gr\hndl\texture
EntityFX zo\screenmesh,gr\fx
;-----------------------------------------------



Next;grass collection




;------------------------------
;make Lookup Tables
Dim lcos#(360)
Dim lsin#(360)

For c=0 To 360
	lsin#(c) = Sin(Float c)
Next
For c=0 To 360
	lcos#(c) = Cos(Float c)
Next
;---------------------------

SetBuffer buf



End Function









;Creates the grass collection

Function Create_grass.girt(id,x#,y#,z#,num,tex$,texflag,fx,blend,alpha#);

If num>21845 Then num=21845;max tri num

	brush=CreateBrush()	
	dust=LoadTexture(tex$,texflag);
	BrushTexture brush,dust
	BrushFX brush,fx
	BrushBlend brush,blend;
	BrushAlpha brush,alpha#
	
	grl.girt=New girt
	grl\id=id
	grl\texture=dust
	grl\mesh=CreateMesh()
	grl\x=x:grl\y=y:grl\z=z
	grl\surf=CreateSurface(grl\mesh,brush)
	

	
	For i=1 To num

	a.adirt=New adirt
	a\link=grl\cache
	grl\cache=a
	s=grl\surf
	
	a\id=id

	gi=AddVertex(s,x,y,z ,0.5,-.4)
	   AddVertex(s,x,y,z   ,1.4,1)
	   AddVertex(s,x,y,z   ,-.4,1)
	AddTriangle s,gi,gi+1,gi+2

	a\index=gi
	a\pos=New vector
	a\pos\id=id

	Next


	FreeBrush brush
	
	Return grl
	
End Function


;add grass to the collection
Function Add_grass(s.girt,size#,size2#,x#,y#,z#)
	a.adirt=s\pos
	If a=Null a=s\cache 
	s\pos=a\link
	
	a\pos\x=x
	a\pos\y=y
	a\pos\z=z

	
	a\size=size
	a\size2=size2
End Function




;This function is called only once to give to the particles-plants
;it's original size and placing
Function Update_grass(grl.girt,terrain,r,g,b)
	
	surf=grl\surf
	
	a.adirt=grl\cache

	
	While a<>Null


			p.vector=a\pos
				
				;variable size or not
				If a\size2=0
				scale#=a\size
				Else
				scale#=Rnd(a\size,a\size2)
				EndIf

				;place randomly along the xy or zy axis
				gfl%=Int(Rnd(0,1))
				If gfl=1
				cxx#=1*scale:cxy#=0*scale:cxz#=0*scale
				cyx#=0*scale:cyy#=-1*scale:cyz#=0*scale
				Else
				cxx#=0*scale:cxy#=0*scale:cxz#=1*scale
				cyx#=0*scale:cyy#=-1*scale:cyz#=0*scale
				EndIf
	

				i=a\index
			
				x#=p\x-cyx
				y#=p\y-cyy
				z#=p\z-cyz
				VertexCoords surf,i,x,y,z

				x=x+cyx-cxx
				y=y+cyy-cxy
				z=z+cyz-cxz
				VertexCoords surf,i+1,x,y,z

				x=x+cxx+cxx
				y=y+cxy+cxy
				z=z+cxz+cxz
				VertexCoords surf,i+2,x,y,z

		a=a\link
	Wend


;rotate randomly across y-axis
For i=0 To CountTriangles(surf)-1
TurnTriangle(surf,i,0,Int(Rnd(0,3600)),0,terrain,grl\mesh)
Next

;vertex color each particle
For i=0 To CountVertices(surf)-1
VertexColor surf,i,r,g,b
Next

	
End Function






;A simple wind function
;It moves the top vertex of the plant-triangle
;Todo-make the vertex not move too far away from it's base

Global wind#

Function Updatewind(grl.girt)

a.adirt=grl\cache

wind#=wind#+1.0
If wind=360.0 wind=0.0
	
While a<>Null
x#=VertexX(grl\surf,a\index)+lsin(wind)/10.0
y#=VertexY(grl\surf,a\index)
z#=VertexZ(grl\surf,a\index)+lcos(wind)/10.0
VertexCoords grl\surf,a\index,x,y,z
a=a\link
Wend
	
End Function










;##########################################################################



;destroys a specific grass collection identified by it's id
Function Destroy_plant(id)

Local gflag=0

gr.grass=First grass:If gr=Null Then Return

For gr.grass=Each grass
If id=gr\id gflag=1:Exit
Next
If gflag=0 Return

gr.grass=First grass
While Not gr\id = id:gr=After gr:Wend
Delete gr

grl.girt=First girt
While Not grl\id = id:grl=After grl:Wend
If grl\mesh<>0 FreeEntity grl\mesh
Delete grl


For a.adirt=Each adirt
If a\id=id Then Delete a
Next

For p.vector=Each vector
If p\id=id Then Delete p
Next

zo.zorder = First zorder
While Not zo\id = id:zo=After zo:Wend
If zo\hiddenmesh<>0 FreeEntity zo\hiddenmesh
FreeBank zo\x
FreeBank zo\z
If zo\incy FreeBank zo\y
Delete zo


End Function







;Destroys all the grass emitters
Function Destroy_grass()

For gr.grass=Each grass
Delete gr
Next

For grl.girt=Each girt
If grl\mesh<>0 FreeEntity grl\mesh
Delete grl
Next

Delete Each adirt

Delete Each vector

Dim lcos#(0),lsin#(0)

For zo.zorder = Each zorder
If zo\hiddenmesh<>0 FreeEntity zo\hiddenmesh
FreeBank zo\x
FreeBank zo\z
If zo\incy FreeBank zo\y
Delete zo
Next

End Function



;By David Bird
;Is used to rotate randomly the plant along the y-axis
Function TurnTriangle(surf,index,ax#,ay#,az#,terrain,mesh)
	Local CosAX#=Cos(ax),SinAX#=Sin(ax)
	Local CosAY#=Cos(ay),SinAY#=Sin(ay)
	Local CosAZ#=Cos(az),SinAZ#=Sin(az)
	Local x#[2],y#[2],z#[2],i[2]
	Local avx#,avy#,avz#
	
	For a=0 To 2
		i[a]=TriangleVertex(surf,index,a)
		x[a]=VertexX(surf,i[a])
		y[a]=VertexY(surf,i[a])
		z[a]=VertexZ(surf,i[a])
		avx=avx+x[a]
		avy=avy+y[a]
		avz=avz+z[a]
	Next
	
	ox#=avx/Float(3)
	oy#=avy/Float(3)
	oz#=avz/Float(3)
	
	For a=0 To 2
		x[a]=x[a]-ox
		y[a]=y[a]-oy
		z[a]=z[a]-oz
		;rotate on x
		ty#=y[a]*cosAX+z[a]*sinAX
		z[a]=z[a]*cosAX-y[a]*sinAX
		y[a]=ty
		;rotate on y
		tx#=x[a]*cosAY+z[a]*sinAY
		z[a]=z[a]*cosAY-x[a]*sinAY
		x[a]=tx
		;rotate on z
		tx#=x[a]*cosAZ+y[a]*sinAZ
		y[a]=y[a]*cosAZ-x[a]*sinAZ
		x[a]=tx
		x[a]=x[a]+ox
		y[a]=y[a]+oy
		z[a]=z[a]+oz
		VertexCoords surf,i[a],x[a],y[a],z[a]

;---------------------------------------------------
;do not allow an edge of a plant to float in the air
TFormPoint x[a],y[a],z[a],mesh,0
jx=TFormedX():jy=TFormedY():jz=TFormedZ()
jjy=TerrainY(terrain,jx,jy,jz)
If a<>0
If jy>jjy VertexCoords surf,i[a],jx,jjy,jz
EndIf
;----------------------------------------------------

	Next
	
End Function



;This function is used to flip vertical the grass-map
;This way you get correct placement on the terrain.
Function flip_vertical(img)
buf=GraphicsBuffer()
temp=CopyImage(img)
w=ImageWidth(temp)-1
h=ImageHeight(temp)-1
LockBuffer(ImageBuffer(img))
LockBuffer(ImageBuffer(temp))
For x=0 To w
For y=0 To h
rgb=ReadPixelFast (x,y,ImageBuffer(temp))
WritePixelFast x,h-y,rgb,ImageBuffer(img)
Next
Next
UnlockBuffer(ImageBuffer(img))
UnlockBuffer(ImageBuffer(temp))
FreeImage temp
SetBuffer buf
Return img
End Function